module DDLParser
  module DDL
    module DB2
      class Parser < Parslet::Parser

        include DDLParser::SharedRules::Constants
        include DDLParser::SharedRules::LogicalOperators
        include DDLParser::SharedRules::DataTypes

        # column options
        rule(:option_not_null)   { str('not null') }
        rule(:column_option)     { (option_not_null).as(:column_option) }
        #rule(:lob_options)
        rule(:primary_key)       { (str('primary key') >> spaces >> references).as(:primary_key) }
        rule(:constraint)        { (str('constraint') >> spaces >> identifier.as(:column_name) >> spaces >> (str('unique') | str('foreign key')).as(:constraint_type) >>
                                   spaces >> references.as(:constraint_arglist) >> spaces >> reference_clause.maybe).as(:constraint)}
        #rule(:unique)
        rule(:references)        { lparen >> space? >> arglist >> space? >> rparen }
        rule(:reference_clause)  { str('references') >> spaces >> identifier >> spaces >> references.as(:reference_arglist) >> (identifier >> spaces).repeat }
        #rule(:check)
        #rule(:check_condition)
        #rule(:constraint_attr)
        rule(:gen_col_def)       { str('generated always') | str('generated by default') }
        rule(:default_value)     { const | current_timestamp | function }
        rule(:default_values)    { (default_value >> space?).repeat }
        rule(:default_clause)    { ((str('with') >> space?).maybe >> str('default') >> space? >> default_values).as(:default_clause) }
        rule(:identity_options)  { (str('as identity') >> space? >> lparen >> str('start with') >> spaces >>
                                   integer.as(:start_value) >> (comma|space) >> spaces.maybe >>str('increment by') >> spaces >> integer.as(:increment_value)>> ((comma|space) >> spaces.maybe >> str('cache')>>spaces>>integer.as(:cache_value)).maybe >> spaces.maybe >>rparen).as(:identity) }

        rule(:column_options)    { ((gen_col_def | column_option | default_clause | identity_options) >> spaces).repeat }

        rule(:column_name)       { identifier }
        rule(:constraint_name)   { identifier }
        rule(:column_sort)       { str('ascending') |str('asc') | str('descending')|str('desc')}

        rule(:column_definition) { (column_name.as(:field) >> space.repeat >> data_type >> space.repeat >> (column_options.maybe).as(:options)).as(:column)}

        rule(:index_column_definition) { column_name.as(:field) >> (space? >> column_sort.as(:sort_type) >> space.repeat).maybe }

        rule(:element_list)      { (lparen >> space? >> (column_definition | constraint | primary_key) >> space? >>
                                    (comma >> space? >> (column_definition | constraint | primary_key) >> space?).repeat >> rparen).as(:elements)
                                 }

        rule(:element_list_index) { space? >> index_column_definition >>
                                    (comma >> space? >> index_column_definition).repeat
                                  }

        #rule(:element_list_index) { (space? >> index_column_definition >>
        #                          (comma >> space? >> index_column_definition).repeat).as(elements)
        #                          }

        rule(:add_constraint_clause)  { (str('add constraint') >> space? >>
                                    constraint_name.as(:constraint_name) >> space? >>
                                    foreign_key_clause.as(:foreign_key_clause)).as(:add_constraint)
                                  }
        rule(:foreign_key_clause)  {str('foreign key') >> space? >> lparen >>
                                    element_list_index.as(:member_fields) >> space? >> rparen >> space?  >>
                                    reference_clause.as(:reference_clause) >> space? >>
                                    (on_delete_clause.as(:on_delete_clause) >> space?).maybe >>
                                    (on_update_clause >> space?).maybe
                                  }
        rule(:on_delete_clause)    {str('on delete') >> space? >>
                                    (str('no action')|str('restrict')|str('cascade')|str('set null')).as(:on_delete_option) >> space?
                                  }
        rule(:on_update_clause)    {str('on update') >> space? >>
            (str('no action')|str('restrict')).as(:on_update_option) >> space?
                                  }

        rule(:alter_table_add_column) do
          (str('add') >> space? >> (str('column') >> space? >> column_definition) | primary_key).as(:add)
        end

        rule(:alter_table_element) { (alter_table_add_column | add_constraint_clause) >> spaces.maybe }

        #rule(:alter_table_alter) { }
        #rule(:alter_table_drop)  { }

        rule(:term)       { const | item }

        rule(:function)   {
          (identifier.as(:name) >> space? >>
              lparen >> arglist.as(:arguments) >> rparen).as(:function)
        }
        # index options

        rule(:option_reverse_scans)   { str('allow reverse scans').as(:reverse_scans_property)}
        rule(:index_option)     { (option_reverse_scans).as(:index_option) }

        rule(:item)       { function | identifier | string }
        rule(:arglist)    {
          item.as(:item) >> (comma >> item.as(:item)).repeat
        }

        rule(:create_table_statement)   {
          str('create table').as(:operation) >> space? >> (identifier).as(:table_name)
        }
        rule(:alter_table_statement)   {
          str('alter table').as(:operation) >> space? >> (identifier).as(:table_name)
        }
        rule(:create_index_statement)   {
          str('create').as(:operation) >> (space? >> str('unique').as(:object_property)).maybe >> space? >> str('index').as(:object_type) >> space? >> (identifier).as(:index_name) >> space?
        }
        rule(:index_table)   {
          space? >> str('on') >> space? >> (identifier).as(:table_name) >> space?
        }
        rule(:create_table)     {
          spaces.maybe >> create_table_statement >> spaces >> element_list >> spaces.maybe >> (item >> spaces.maybe).repeat
        }
        rule(:alter_table)     {
          spaces.maybe >> alter_table_statement >> spaces >> (alter_table_element.repeat).as(:elements)
        }
        rule(:create_index)     {
          spaces.maybe >> create_index_statement >> spaces >> index_table >> spaces >> lparen >> element_list_index >> spaces.maybe >> rparen  >> (space? >> index_option).maybe >> space?
        }
        rule(:expression) { create_table | alter_table | create_index}
        root :expression
      end
    end
  end
end

